import { Creator, Factory, ResolutionCondition, UnknownConstructor, UnknownFunction } from '../../types';
import { Token } from '../../pointers';
import { BindingsVault } from '../BindingsVault';
import { ScopeSyntax } from './ScopeSyntax';
export declare class TypeSyntax<T> {
    private readonly bindingsVault;
    private readonly token;
    private readonly condition?;
    constructor(bindingsVault: BindingsVault, token: Token, condition?: ResolutionCondition | undefined);
    toInstance<K extends UnknownConstructor<T>>(ctor: K): ScopeSyntax;
    toCall<K extends UnknownFunction<T>>(func: K): ScopeSyntax;
    toConstant(value: T): void;
    /**
     * @example <caption>Example usage of factory without arguments.</caption>
     * const someClassFactoryToken = token<Factory<SomeClass>>('someClassFactory');
     *
     * container
     * .bind(someClassFactoryToken)
     * .toFactory(SomeClass);
     * // OR
     * container
     * .bind(someClassFactoryToken)
     * .toFactory(SomeClass, (instance) => instance.init());
     *
     * const someClassFactory = container.get(someClassFactoryToken);
     * const someClassInstance = someClassFactory();
     *
     * console.log(someClassInstance instanceof SomeClass); // -> true
     */
    toFactory(ctor: T extends Factory<infer R> ? R extends Object ? UnknownConstructor<R> : never : never, initializer?: T extends Factory<infer R> ? (instance: R) => unknown : never): void;
    /**
     * @example <caption>Example usage of factory with arguments.</caption>
     * const someClassFactoryToken = token<Factory<SomeClass, [name: string]>>('someClassFactory');
     *
     * container
     * .bind(someClassFactoryToken)
     * .toFactory(SomeClass, (instance, name) => instance.setName(name));
     *
     * const someClassFactory = container.get(someClassFactoryToken);
     * const someClassInstance = someClassFactory('Olivia');
     *
     * console.log(someClassInstance instanceof SomeClass); // -> true
     */
    toFactory(ctor: T extends Factory<infer R, never[]> ? R extends Object ? UnknownConstructor<R> : never : never, initializer: T extends Factory<infer R, infer A> ? (instance: R, ...args: A) => unknown : never): void;
    /**
     * @example <caption>Example usage of creator without arguments.</caption>
     * function createSomeEntity(): SomeEntity;
     *
     * const someEntityCreatorToken = token<Creator<SomeEntity>>('someEntityCreator');
     *
     * container
     * .bind(someEntityCreatorToken)
     * .toCreator(createSomeEntity);
     * // OR
     * container
     * .bind(someEntityCreatorToken)
     * .toCreator(createSomeEntity, (entity) => entity.init());
     *
     * const someEntityCreator = container.get(someEntityCreatorToken);
     * const someEntity = someEntityCreator();
     *
     * type ReturnedEntity = typeof someEntity; // to be SomeEntity
     */
    toCreator(func: T extends Creator<infer R> ? UnknownFunction<R> : never, initializer?: T extends Creator<infer R> ? (entity: R) => unknown : never): void;
    /**
     * @example <caption>Example usage of creator with arguments.</caption>
     * function createSomeEntity(): SomeEntity;
     *
     * const someEntityCreatorToken = token<Creator<SomeEntity, [name: string]>>('someEntityCreator');
     *
     * container
     * .bind(someEntityCreatorToken)
     * .toCreator(createSomeEntity, (entity, name) => entity.setName(name));
     *
     * const someEntityCreator = container.get(someEntityCreatorToken);
     * const someEntity = someEntityCreator('Olivia');
     *
     * type ReturnedEntity = typeof someEntity; // to be SomeEntity
     */
    toCreator(func: T extends Creator<infer R, never[]> ? UnknownFunction<R> : never, initializer: T extends Creator<infer R, infer A> ? (entity: R, ...args: A) => unknown : never): void;
}
